import pytest
import json
import os

from tests.conftest import VirtualEnvironment, clear_user_json

from mitoinstaller.user_install import get_static_user_id, try_create_user_json_file
from mitoinstaller.user_install import USER_JSON_PATH


def get_mitosheet_version_in_user_json():
    try:
        return json.loads(open(USER_JSON_PATH, 'r').read())['mitosheet_current_version']
    except:
        return None


def test_import_mitosheet_keeps_same_user_id(venv: VirtualEnvironment, clear_user_json):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])    
    # First, check there is no user.json
    static_user_id_installer = get_static_user_id()
    assert static_user_id_installer is None
    venv.run_python_module_command(['mitoinstaller', 'install'])

    # We get the user id created by the installer
    static_user_id_installer = get_static_user_id()
    assert static_user_id_installer is not None
    mitosheet_version = get_mitosheet_version_in_user_json()
    assert mitosheet_version is None
    
    # Then, we import Mito
    venv.run_python_script('import mitosheet')
    
    # And check that the static id has not changed, but the rest
    # of the user.json file has 
    static_user_id_new = get_static_user_id()
    assert static_user_id_installer == static_user_id_new
    mitosheet_version = get_mitosheet_version_in_user_json()
    assert mitosheet_version is not None

def test_installer_does_not_overwrite_static_user_id(venv: VirtualEnvironment, clear_user_json):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])    
    # First, check there is no user.json
    static_user_id_installer = get_static_user_id()
    assert static_user_id_installer is None

    # Then, create an id
    try_create_user_json_file()
    static_user_id_before_installer = get_static_user_id()

    # Then, run the installer
    venv.run_python_module_command(['mitoinstaller', 'install'])

    # We get the user id created by the installer
    static_user_id_installer = get_static_user_id()
    assert static_user_id_installer is not None
    mitosheet_version = get_mitosheet_version_in_user_json()
    assert mitosheet_version is None
    
    # Then, we import Mito
    venv.run_python_script('import mitosheet')
    
    # And check that the static id has not changed, but the rest
    # of the user.json file has 
    static_user_id_new = get_static_user_id()
    assert static_user_id_before_installer == static_user_id_installer
    assert static_user_id_installer == static_user_id_new
    mitosheet_version = get_mitosheet_version_in_user_json()
    assert mitosheet_version is not None


def test_running_installer_twice_does_not_overwrite_static_user_id(venv: VirtualEnvironment, clear_user_json):
    venv.run_python_module_command(['pip', 'install', '-r', 'requirements.txt'])    
    # First, check there is no user.json
    static_user_id_installer = get_static_user_id()
    assert static_user_id_installer is None

    # Then, run the installer
    venv.run_python_module_command(['mitoinstaller', 'install'])
    static_user_id_one = get_static_user_id()

    venv.run_python_module_command(['mitoinstaller', 'install'])
    static_user_id_two = get_static_user_id()

    assert static_user_id_one == static_user_id_two


import os
import sys

from mitoinstaller import __version__
from mitoinstaller.commands import upgrade_mito_installer
from mitoinstaller.installer_steps.installer_step import InstallerStep
from mitoinstaller.log_utils import identify, log
from mitoinstaller.user_install import (USER_JSON_PATH,
                                        try_create_user_json_file)


def initial_install_step_create_user():

    if not os.path.exists(USER_JSON_PATH):
        try_create_user_json_file(is_pro=('--pro' in sys.argv))

    # Only try and log if we're not pro
    if not ('--pro' in sys.argv):
        identify()
        log('install_started', {
            'mitoinstaller_version': __version__
        })


INITIAL_INSTALLER_STEPS = [
    InstallerStep(
        'Create mito user',
        initial_install_step_create_user
    ),
    InstallerStep(
        'Upgrade mitoinstaller',
        upgrade_mito_installer,
        optional=True
    ),
]



from distutils.core import setup
from mitoinstaller import __version__
from setuptools import find_packages

setup(
    name='mitoinstaller',
    version=__version__,
    packages=find_packages(where='.', exclude='tests'),
    install_requires=['analytics-python', 'colorama', 'termcolor'],
    license='AGPL-3.0-only',
    long_description='The mitoinstaller package allows you to install the mitosheet package in your local JupyterLab instance. It logs anonymous data about installation, including if it is successful or why it failed.',
)



import sys
from time import perf_counter
from mitoinstaller.commands import (get_jupyterlab_metadata,
                                    install_pip_packages,
                                    uninstall_pip_packages)
from mitoinstaller.installer_steps.installer_step import InstallerStep
from mitoinstaller.log_utils import log


def install_step_mitosheet_check_dependencies():
    """
    This is the most complex step in the installation process. It's
    goal is to check if the users existing installation can safely
    be upgraded to JLab 3.0. 

    To do this, it checks a variety of conditions, mostly around what
    version of JLab they have installed, and if this version of JLab has
    any installed dependencies (that we cannot safely upgrade).
    """

    jupyterlab_version, extension_names = get_jupyterlab_metadata()

    # If no JupyterLab is installed, we can continue with install, as
    # there are no conflict dependencies
    if jupyterlab_version is None:
        return
    
    # If JupyterLab 3 is installed, then we are are also good to go
    if jupyterlab_version.startswith('3'):
        return

    # If the user has no extension, we can upgrade
    if len(extension_names) == 0:
        return
    
    else:
        raise Exception('Installed extensions {extension_names}'.format(extension_names=extension_names))

def remove_mitosheet_3_if_present():
    """
    Because of our changes to the package hierarchy, if the user
    currently has mitosheet3 installed, we need to uninstall it,
    as the user is moving to mitosheet instead. 

    Otherwise, if the user has mitosheet and mitosheet3 installed,
    they will get an error launching jlab because it will lead to 
    two different extensions getting registered with the same
    name.
    """
    uninstall_pip_packages('mitosheet3')
    

def install_step_mitosheet_install_mitosheet():
    install_pip_packages('mitosheet', test_pypi='--test-pypi' in sys.argv)

    
MITOSHEET_INSTALLER_STEPS = [
    InstallerStep(
        'Check dependencies',
        install_step_mitosheet_check_dependencies
    ),
    InstallerStep(
        'Remove mitosheet3 if present',
        remove_mitosheet_3_if_present
    ),
    InstallerStep(
        'Install mitosheet',
        install_step_mitosheet_install_mitosheet,
        should_log_success=True
    ),
]
